==> jijzept-1.14.0rc7-cp39-cp39-win_amd64.whl: jijzept/sampler/ms_qio/ms_qio.py <==
from __future__ import annotations

import dataclasses
import typing as typ

from numbers import Number

import jijmodeling as jm
import numpy as np

from jijzept.entity.schema import SolverType
from jijzept.response import JijModelingResponse
from jijzept.sampler.base_sampler import (
    JijZeptBaseSampler,
    ParameterSearchParameters,
    merge_params_and_kwargs,
    sample_model,
)
from jijzept.sampler.sample_hubo import sample_hubo
from jijzept.sampler.sample_qubo import sample_qubo

T = typ.TypeVar("T")


class QIOParameters:
    pass


P = typ.TypeVar("P")


class JijQIOBaseSampler(JijZeptBaseSampler, typ.Generic[P]):
    parameter_cls: typ.Type[QIOParameters]

    def sample_model(
        self,
        model: jm.Problem,
        feed_dict: dict[str, Number | list | np.ndarray],
        multipliers: dict[str, Number] = {},
        fixed_variables: dict[str, dict[tuple[int, ...], int | float]] = {},
        search: bool = False,
        num_search: int = 15,
        algorithm: str | None = None,
        parameters: P | None = None,
        timeout: int | None = None,
        sync: bool = True,
        queue_name: str | None = None,
        **kwargs,
    ) -> JijModelingResponse:
        """Sample using JijModeling by means of Microsoft QIO Simulated Annealing solver.

        Args:
            model (jijmodeling.Problem): Mathematical model is constracted by JijModeling.
            feed_dict (Dict[str, Union[Number, list, np.ndarray]]): The actual values to be assigned to the
            placeholders.
            multipliers (Dict[str, Number], optional): The actual multipliers for penalty terms, derived from
            constraint conditions.
            fixed_variables (Dict[str, Dict[Tuple[int, ...], Union[int, float]]]): dictionary of variables to fix.
            search (bool, optional): If `True`, the parameter search will be carried out, which tries to find better
            values of multipliers for penalty terms.
            num_search (int, optional): The number of parameter search iteration. Defaults to set 15. This option
            works if `search` is `True`.
            algorithm (Optional[str]): Algorithm for parameter search. Defaults to None.
            parameters: (P | None, optional): parameters for QIO. Defaults to None.
            timeout (Optional[int], optional): The number of timeout [sec] for post request. If `None`,
            3600 (one hour) will be set.
            sync (bool, optional): Synchronous mode.

        Returns:
            JijModelingResponse: Stores minimum energy samples and other information.

        Examples:
            ```python
            import jijzept as jz
            import jijmodeling as jm
            n = jm.Placeholder('n')
            x = jm.Binary('x', shape=n)
            d = jm.Placeholder('d', shape=n)
            i = jm.Element("i", n)
            problem = jm.Problem('problem')
            problem += jm.Sum(i, d[i] * x[i])
            sampler = jz.JijQIOSASampler(config='config.toml')
            response = sampler.sample_model(problem, feed_dict={'n': 5, 'd': [1,2,3,4,5]})
            ```
        """
        param_dict = merge_params_and_kwargs(parameters, kwargs, self.parameter_cls)

        if queue_name is None:
            queue_name = self.jijmodeling_solver_type.queue_name

        para_search_params = ParameterSearchParameters(
            multipliers=multipliers,
            mul_search=search,
            num_search=num_search,
            algorithm=algorithm,
        )

        sample_set = sample_model(
            self.client,
            self.jijmodeling_solver_type.solver,
            queue_name=queue_name,
            problem=model,
            instance_data=feed_dict,
            fixed_variables=fixed_variables,
            parameter_search_parameters=para_search_params,
            timeout=timeout,
            sync=sync,
            **param_dict,
        )
        return sample_set

    def sample_qubo(
        self,
        Q: dict,
        parameters: P | None = None,
        timeout: int | None = None,
        sync: bool = True,
        queue_name: str | None = None,
        **kwargs,
    ) -> JijModelingResponse:
        """Microsoft QIO solver is performed for binary quadratic models.

        Args:
            Q (Dict): The linear or quadratic terms.
            num_sweeps (Optional[int], optional): The number of Monte-Carlo steps. If `None`, this will be set
            automatically.
            beta_min (Optional[float], optional): Minimum (initial) inverse temperature. If `None`, this will be set
            automatically.
            beta_max (Optional[float], optional): Maximum (final) inverse temperature. If `None`, this will be set
            automatically.
            num_reads (Optional[int], optional): The number of samples. If `None`, this will be set automatically.
            seed (Optional[int], optional): Seed for Monte Carlo algorithm. If `None`, this will be set automatically.
            platform (Optional[str]): "CPU" or "FPGA". If `None`, "CPU" will be set.
            timeout (Optional[int], optional): The number of timeout [sec] for post request. If `None`,
            3600 (one hour) will be set.
            sync (bool, optional): Synchronous mode.

        Returns:
            dimod.SampleSet: Stores minimum energy samples and other information.

        Examples:
            ```python
            import jijzept as jz
            sampler = jz.JijQIOSASampler(config='config.toml')
            response = sampler.sample_qubo(Q={(0,0): -0.5, (0,1): -1, (0,2): -1, (2,3): -1})
            ```
        """

        param_dict = merge_params_and_kwargs(parameters, kwargs, self.parameter_cls)

        if queue_name is None:
            queue_name = self.jijmodeling_solver_type.queue_name

        sample_set = sample_qubo(
            self.client,
            self.jijmodeling_solver_type.solver,
            queue_name=queue_name,
            qubo=Q,
            constant=0,
            timeout=timeout,
            sync=sync,
            **param_dict,
        )

        return sample_set

    def sample_hubo(
        self,
        J: dict,
        parameters: P | None = None,
        timeout: int | None = None,
        sync: bool = True,
        queue_name: str | None = None,
        **kwargs,
    ) -> JijModelingResponse:
        """Microsoft QIO simulated annealing solver is performed for binary polynomial models.

        Args:
            J (Dict): Polynomial interactions.
            vartype (str): The variable type. "SPIN" or "BINARY".
            num_sweeps (Optional[int], optional): The number of Monte-Carlo steps. If `None`, this will be set
            automatically.
            beta_min (Optional[float], optional): Minimum (initial) inverse temperature. If `None`, this will be set
            automatically.
            beta_max (Optional[float], optional): Maximum (final) inverse temperature. If `None`, this will be set
            automatically.
            num_reads (Optional[int], optional): The number of samples. If `None`, this will be set automatically.
            seed (Optional[int], optional): Seed for Monte Carlo algorithm. If `None`, this will be set automatically.
            platform (Optional[str]): "CPU" or "FPGA". If `None`, "CPU" will be set.
            timeout (Optional[int], optional): The number of timeout [sec] for post request. If `None`,
            3600 (one hour) will be set.
            sync (bool, optional): Synchronous mode.

        Returns:
            dimod.SampleSet: Stores minimum energy samples and other information.

        Examples:
            ```python
            import jijzept as jz
            sampler = jz.JijQIOSASampler(config='config.toml')
            response = sampler.sample_hubo(J={(0,1,2,3,4): -1}, vartype="SPIN")
            ```
        """
        param_dict = merge_params_and_kwargs(parameters, kwargs, self.parameter_cls)
        if queue_name is None:
            queue_name = self.jijmodeling_solver_type.queue_name

        sample_set = sample_hubo(
            self.client,
            self.jijmodeling_solver_type.solver,
            queue_name=queue_name,
            polynomial=J,
            constant=J.get((), 0.0),
            timeout=timeout,
            sync=sync,
            **param_dict,
        )
        return sample_set


@dataclasses.dataclass
class JijQIOSAParameters(QIOParameters):
    num_sweeps: int | None = None
    beta_min: float | None = None
    beta_max: float | None = None
    num_reads: int | None = None
    seed: int | None = None
    platform: str | None = None


class JijQIOSASampler(JijQIOBaseSampler[JijQIOSAParameters]):
    solver_type: SolverType = SolverType(queue_name="qiosolver", solver="SA")
    jijmodeling_solver_type = SolverType(queue_name="qiosolver", solver="SAParaSearch")
    parameter_cls: typ.Type[QIOParameters] = JijQIOSAParameters


@dataclasses.dataclass
class JijQIOPAParameters(QIOParameters):
    num_sweeps: int | None = None
    beta_min: float | None = None
    beta_max: float | None = None
    population: int | None = None
    schedule_type: str | None = None
    seed: int | None = None


class JijQIOPASampler(JijQIOBaseSampler[JijQIOPAParameters]):
    solver_type: SolverType = SolverType(queue_name="qiosolver", solver="PA")
    jijmodeling_solver_type = SolverType(queue_name="qiosolver", solver="PAParaSearch")
    parameter_cls = JijQIOPAParameters


@dataclasses.dataclass
class JijQIOPTParameters(QIOParameters):
    num_sweeps: int | None = None
    replicas: int | None = None
    all_betas: list[int | float] | None = None
    seed: int | None = None


class JijQIOPTSampler(JijQIOBaseSampler[JijQIOPTParameters]):
    solver_type: SolverType = SolverType(queue_name="qiosolver", solver="PT")
    jijmodeling_solver_type = SolverType(queue_name="qiosolver", solver="PTParaSearch")
    parameter_cls: typ.Type[QIOParameters] = JijQIOPTParameters


@dataclasses.dataclass
class JijQIOTBParameters(QIOParameters):
    num_sweeps: int | None = None
    tabu_tenure: int | None = None
    num_reads: int | None = None
    seed: int | None = None


class JijQIOTBSampler(JijQIOBaseSampler):
    solver_type: SolverType = SolverType(queue_name="qiosolver", solver="TB")
    jijmodeling_solver_type = SolverType(queue_name="qiosolver", solver="TBParaSearch")
    parameter_cls: typ.Type[QIOParameters] = JijQIOTBParameters


@dataclasses.dataclass
class JijQIOSQAParameters(QIOParameters):
    num_sweeps: int | None = None
    num_reads: int | None = None
    beta: float | None = None
    trotter: int | None = None
    transverse_field_max: float | None = None
    transverse_field_min: float | None = None
    seed: int | None = None


class JijQIOSQASampler(JijQIOBaseSampler):
    solver_type: SolverType = SolverType(queue_name="qiosolver", solver="SQA")
    jijmodeling_solver_type = SolverType(queue_name="qiosolver", solver="SQAParaSearch")
    parameter_cls: typ.Type[QIOParameters] = JijQIOSQAParameters


@dataclasses.dataclass
class JijQIOSSMCParameters(QIOParameters):
    search: bool = False
    num_search: int = 15
    algorithm: str | None = None
    step_limit: int | None = None
    target_population: int | None = None
    alpha_min: float | None = None
    alpha_max: float | None = None
    beta_min: float | None = None
    beta_max: float | None = None
    alpha_schedule_type: str | None = None
    beta_schedule_type: str | None = None
    seed: int | None = None


class JijQIOSSMCSampler(JijQIOBaseSampler[JijQIOSSMCParameters]):
    solver_type: SolverType = SolverType(queue_name="qiosolver", solver="SSMC")
    jijmodeling_solver_type = SolverType(
        queue_name="qiosolver", solver="SSMCParaSearch"
    )
    parameter_cls: typ.Type[QIOParameters] = JijQIOSSMCParameters

==> jijzept-1.15.0rc1-cp39-cp39-win_amd64.whl: jijzept/sampler/ms_qio/ms_qio.py <==
from __future__ import annotations

import dataclasses
import typing as typ

from numbers import Number

import jijmodeling as jm
import numpy as np

from jijzept.entity.schema import SolverType
from jijzept.response import JijModelingResponse
from jijzept.sampler.base_sampler import (
    JijZeptBaseSampler,
    ParameterSearchParameters,
    merge_params_and_kwargs,
    sample_model,
)
from jijzept.sampler.sample_hubo import sample_hubo
from jijzept.sampler.sample_qubo import sample_qubo
from jijzept.sampler.utils import _set_needs_square

T = typ.TypeVar("T")


class QIOParameters:
    pass


P = typ.TypeVar("P")


class JijQIOBaseSampler(JijZeptBaseSampler, typ.Generic[P]):
    parameter_cls: typ.Type[QIOParameters]

    def sample_model(
        self,
        model: jm.Problem,
        feed_dict: dict[str, Number | list | np.ndarray],
        multipliers: dict[str, Number] = {},
        fixed_variables: dict[str, dict[tuple[int, ...], int | float]] = {},
        needs_square_dict: dict[str, bool] | None = None,
        search: bool = False,
        num_search: int = 15,
        algorithm: str | None = None,
        parameters: P | None = None,
        max_wait_time: int | float | None = None,
        sync: bool = True,
        queue_name: str | None = None,
        **kwargs,
    ) -> JijModelingResponse:
        """Sample using JijModeling by means of Microsoft QIO Simulated Annealing solver.

        To configure the solver, instantiate the `JijQIOXXParameters` class for your desired solver and pass the instance to the `parameters` argument.

        Args:
            model (jijmodeling.Problem): Mathematical model is constracted by JijModeling.
            feed_dict (Dict[str, Union[Number, list, np.ndarray]]): The actual values to be assigned to the
            placeholders.
            multipliers (Dict[str, Number], optional): The actual multipliers for penalty terms, derived from
            constraint conditions.
            fixed_variables (Dict[str, Dict[Tuple[int, ...], Union[int, float]]]): dictionary of variables to fix.
            needs_square_dict (Dict[str, bool], optional): If `True`, the corresponding constraint is squared when added as a penalty to the QUBO.
            When the constraint label is not added to the 'needs_square_dict', it defaults to `True` for linear constraints and `False` for non-linear constraints.
            search (bool, optional): If `True`, the parameter search will be carried out, which tries to find better
            values of multipliers for penalty terms.
            num_search (int, optional): The number of parameter search iteration. Defaults to set 15. This option
            works if `search` is `True`.
            algorithm (Optional[str]): Algorithm for parameter search. Defaults to None.
            parameters: (P | None, optional): parameters for QIO. Defaults to None.
            max_wait_time (int | float | None, optional): The number of timeout [sec] for post request. If `None`, 3600 (one hour) will be set.
            Please note that this argument is for the `jijzept` timeout and not for configuring solver settings, such as solving time.
            sync (bool, optional): Synchronous mode.

        Returns:
            JijModelingResponse: Stores minimum energy samples and other information.

        Examples:
            ```python
            import jijzept as jz
            import jijmodeling as jm
            n = jm.Placeholder('n')
            x = jm.Binary('x', shape=n)
            d = jm.Placeholder('d', shape=n)
            i = jm.Element("i", n)
            problem = jm.Problem('problem')
            problem += jm.Sum(i, d[i] * x[i])
            sampler = jz.JijQIOSASampler(config='config.toml')
            response = sampler.sample_model(problem, feed_dict={'n': 5, 'd': [1,2,3,4,5]})
            ```
        """
        model = _set_needs_square(model, needs_square_dict)
        param_dict = merge_params_and_kwargs(parameters, kwargs, self.parameter_cls)

        if queue_name is None:
            queue_name = self.jijmodeling_solver_type.queue_name

        para_search_params = ParameterSearchParameters(
            multipliers=multipliers,
            mul_search=search,
            num_search=num_search,
            algorithm=algorithm,
        )

        sample_set = sample_model(
            self.client,
            self.jijmodeling_solver_type.solver,
            queue_name=queue_name,
            problem=model,
            instance_data=feed_dict,
            fixed_variables=fixed_variables,
            parameter_search_parameters=para_search_params,
            max_wait_time=max_wait_time,
            sync=sync,
            **param_dict,
        )
        return sample_set

    def sample_qubo(
        self,
        Q: dict,
        parameters: P | None = None,
        max_wait_time: int | float | None = None,
        sync: bool = True,
        queue_name: str | None = None,
        **kwargs,
    ) -> JijModelingResponse:
        """Microsoft QIO solver is performed for binary quadratic models.

        To configure the solver, instantiate the `JijQIOXXParameters` class for your desired solver and pass the instance to the `parameters` argument.

        Args:
            Q (Dict): The linear or quadratic terms.
            num_sweeps (Optional[int], optional): The number of Monte-Carlo steps. If `None`, this will be set
            automatically.
            beta_min (Optional[float], optional): Minimum (initial) inverse temperature. If `None`, this will be set
            automatically.
            beta_max (Optional[float], optional): Maximum (final) inverse temperature. If `None`, this will be set
            automatically.
            num_reads (Optional[int], optional): The number of samples. If `None`, this will be set automatically.
            seed (Optional[int], optional): Seed for Monte Carlo algorithm. If `None`, this will be set automatically.
            platform (Optional[str]): "CPU" or "FPGA". If `None`, "CPU" will be set.
            max_wait_time (int | float | None, optional): The number of timeout [sec] for post request. If `None`, 3600 (one hour) will be set.
            Please note that this argument is for the `jijzept` timeout and not for configuring solver settings, such as solving time.
            sync (bool, optional): Synchronous mode.

        Returns:
            dimod.SampleSet: Stores minimum energy samples and other information.

        Examples:
            ```python
            import jijzept as jz
            sampler = jz.JijQIOSASampler(config='config.toml')
            response = sampler.sample_qubo(Q={(0,0): -0.5, (0,1): -1, (0,2): -1, (2,3): -1})
            ```
        """

        param_dict = merge_params_and_kwargs(parameters, kwargs, self.parameter_cls)

        if queue_name is None:
            queue_name = self.jijmodeling_solver_type.queue_name

        sample_set = sample_qubo(
            self.client,
            self.jijmodeling_solver_type.solver,
            queue_name=queue_name,
            qubo=Q,
            constant=0,
            max_wait_time=max_wait_time,
            sync=sync,
            **param_dict,
        )

        return sample_set

    def sample_hubo(
        self,
        J: dict,
        parameters: P | None = None,
        max_wait_time: int | float | None = None,
        sync: bool = True,
        queue_name: str | None = None,
        **kwargs,
    ) -> JijModelingResponse:
        """Microsoft QIO simulated annealing solver is performed for binary polynomial models.

        To configure the solver, instantiate the `JijQIOXXParameters` class for your desired solver and pass the instance to the `parameters` argument.

        Args:
            J (Dict): Polynomial interactions.
            vartype (str): The variable type. "SPIN" or "BINARY".
            num_sweeps (Optional[int], optional): The number of Monte-Carlo steps. If `None`, this will be set
            automatically.
            beta_min (Optional[float], optional): Minimum (initial) inverse temperature. If `None`, this will be set
            automatically.
            beta_max (Optional[float], optional): Maximum (final) inverse temperature. If `None`, this will be set
            automatically.
            num_reads (Optional[int], optional): The number of samples. If `None`, this will be set automatically.
            seed (Optional[int], optional): Seed for Monte Carlo algorithm. If `None`, this will be set automatically.
            platform (Optional[str]): "CPU" or "FPGA". If `None`, "CPU" will be set.
            max_wait_time (int | float | None, optional): The number of timeout [sec] for post request. If None, 3600 (one hour) will be set.
            Please note that this argument is for the jijzept timeout and not for configuring solver settings, such as solving time.
            sync (bool, optional): Synchronous mode.

        Returns:
            dimod.SampleSet: Stores minimum energy samples and other information.

        Examples:
            ```python
            import jijzept as jz
            sampler = jz.JijQIOSASampler(config='config.toml')
            response = sampler.sample_hubo(J={(0,1,2,3,4): -1}, vartype="SPIN")
            ```
        """
        param_dict = merge_params_and_kwargs(parameters, kwargs, self.parameter_cls)
        if queue_name is None:
            queue_name = self.jijmodeling_solver_type.queue_name

        sample_set = sample_hubo(
            self.client,
            self.jijmodeling_solver_type.solver,
            queue_name=queue_name,
            polynomial=J,
            constant=J.get((), 0.0),
            max_wait_time=max_wait_time,
            sync=sync,
            **param_dict,
        )
        return sample_set


@dataclasses.dataclass
class JijQIOSAParameters(QIOParameters):
    num_sweeps: int | None = None
    beta_min: float | None = None
    beta_max: float | None = None
    num_reads: int | None = None
    seed: int | None = None
    platform: str | None = None


class JijQIOSASampler(JijQIOBaseSampler[JijQIOSAParameters]):
    solver_type: SolverType = SolverType(queue_name="qiosolver", solver="SA")
    jijmodeling_solver_type = SolverType(queue_name="qiosolver", solver="SAParaSearch")
    parameter_cls: typ.Type[QIOParameters] = JijQIOSAParameters


@dataclasses.dataclass
class JijQIOPAParameters(QIOParameters):
    num_sweeps: int | None = None
    beta_min: float | None = None
    beta_max: float | None = None
    population: int | None = None
    schedule_type: str | None = None
    seed: int | None = None


class JijQIOPASampler(JijQIOBaseSampler[JijQIOPAParameters]):
    solver_type: SolverType = SolverType(queue_name="qiosolver", solver="PA")
    jijmodeling_solver_type = SolverType(queue_name="qiosolver", solver="PAParaSearch")
    parameter_cls = JijQIOPAParameters


@dataclasses.dataclass
class JijQIOPTParameters(QIOParameters):
    num_sweeps: int | None = None
    replicas: int | None = None
    all_betas: list[int | float] | None = None
    seed: int | None = None


class JijQIOPTSampler(JijQIOBaseSampler[JijQIOPTParameters]):
    solver_type: SolverType = SolverType(queue_name="qiosolver", solver="PT")
    jijmodeling_solver_type = SolverType(queue_name="qiosolver", solver="PTParaSearch")
    parameter_cls: typ.Type[QIOParameters] = JijQIOPTParameters


@dataclasses.dataclass
class JijQIOTBParameters(QIOParameters):
    num_sweeps: int | None = None
    tabu_tenure: int | None = None
    num_reads: int | None = None
    seed: int | None = None


class JijQIOTBSampler(JijQIOBaseSampler):
    solver_type: SolverType = SolverType(queue_name="qiosolver", solver="TB")
    jijmodeling_solver_type = SolverType(queue_name="qiosolver", solver="TBParaSearch")
    parameter_cls: typ.Type[QIOParameters] = JijQIOTBParameters


@dataclasses.dataclass
class JijQIOSQAParameters(QIOParameters):
    num_sweeps: int | None = None
    num_reads: int | None = None
    beta: float | None = None
    trotter: int | None = None
    transverse_field_max: float | None = None
    transverse_field_min: float | None = None
    seed: int | None = None


class JijQIOSQASampler(JijQIOBaseSampler):
    solver_type: SolverType = SolverType(queue_name="qiosolver", solver="SQA")
    jijmodeling_solver_type = SolverType(queue_name="qiosolver", solver="SQAParaSearch")
    parameter_cls: typ.Type[QIOParameters] = JijQIOSQAParameters


@dataclasses.dataclass
class JijQIOSSMCParameters(QIOParameters):
    search: bool = False
    num_search: int = 15
    algorithm: str | None = None
    step_limit: int | None = None
    target_population: int | None = None
    alpha_min: float | None = None
    alpha_max: float | None = None
    beta_min: float | None = None
    beta_max: float | None = None
    alpha_schedule_type: str | None = None
    beta_schedule_type: str | None = None
    seed: int | None = None


class JijQIOSSMCSampler(JijQIOBaseSampler[JijQIOSSMCParameters]):
    solver_type: SolverType = SolverType(queue_name="qiosolver", solver="SSMC")
    jijmodeling_solver_type = SolverType(
        queue_name="qiosolver", solver="SSMCParaSearch"
    )
    parameter_cls: typ.Type[QIOParameters] = JijQIOSSMCParameters
